import { createRoot } from "react-dom/client";
import { lazy, Suspense } from "react";
import "./index.css";

/**
 * Performance Utilities - Lightweight inline version to avoid imports
 */
const mark = (name: string) => {
  if (typeof performance !== 'undefined') {
    performance.mark(name);
    console.log(`[Performance] ${name}`);
  }
};

// Mark initial load immediately
mark('app-initialize-start');

/**
 * Feature detection - Check for slow connections to reduce animations
 */
const detectSlowConnection = () => {
  // Check for Save-Data header or slow connection
  if (!navigator.connection) return false;
  
  // Cast to any to avoid TypeScript errors with the connection property
  const conn = navigator.connection as any;
  return conn.saveData || 
    (conn.effectiveType && 
     (conn.effectiveType.includes('2g') || conn.effectiveType.includes('slow')));
};

const hasPoorConnection = detectSlowConnection();

/**
 * Lazy load the main App component
 * This reduces the initial JavaScript payload by splitting the code
 */
const App = lazy(() => {
  mark('app-load-start');
  return import("./App").then(module => {
    mark('app-load-complete');
    return module;
  });
});

/**
 * Simple loading indicator while the app is loading
 * This appears only briefly during the initial load
 */
const AppLoader = () => (
  <div className="initial-loader">
    <div className="loader-spinner"></div>
    <div className="loader-text">Loading Gamut Smith...</div>
  </div>
);

/**
 * Inject critical CSS directly into the head
 * This avoids render-blocking CSS for improved performance
 */
const injectCriticalStyles = () => {
  const style = document.createElement('style');
  style.textContent = `
    /* Critical styles for immediate rendering */
    :root {
      --primary: #ff6347;
      --primary-hover: #ff8c69;
      --background: #1c1c1e;
      --text: #ffffff;
    }
    
    /* Performance optimizations */
    img {
      content-visibility: auto;
    }
    
    /* Smooth scrolling (disabled on poor connections) */
    html {
      scroll-behavior: ${hasPoorConnection ? 'auto' : 'smooth'};
    }
    
    /* Content-visibility for large sections */
    section {
      content-visibility: auto;
      contain-intrinsic-size: 0 500px;
    }
    
    /* Font system optimization */
    body {
      font-family: system-ui, -apple-system, BlinkMacSystemFont, 'Segoe UI', sans-serif;
      text-rendering: optimizeSpeed;
    }
    
    /* Loader styles */
    .initial-loader {
      display: flex;
      flex-direction: column;
      align-items: center;
      justify-content: center;
      height: 100vh;
      background-color: var(--background);
      color: var(--text);
    }
    
    .loader-spinner {
      width: 40px;
      height: 40px;
      border: 3px solid rgba(255, 99, 71, 0.3);
      border-radius: 50%;
      border-top-color: var(--primary);
      animation: ${hasPoorConnection ? 'none' : 'spin 1s ease-in-out infinite'};
      margin-bottom: 1rem;
    }
    
    @keyframes spin {
      to { transform: rotate(360deg); }
    }
    
    .loader-text {
      font-size: 1rem;
      color: var(--text);
    }
    
    /* Layout containment for improved paint performance */
    .contain-layout {
      contain: layout;
    }
    
    /* Paint containment for improved rendering performance */
    .contain-paint {
      contain: paint;
    }
    
    /* Style containment for improved style calculation performance */
    .contain-style {
      contain: style;
    }
    
    /* Full containment for maximum performance */
    .contain-full {
      contain: layout style paint;
    }
    
    /* Disable animations for users who prefer reduced motion or on slow connections */
    @media (prefers-reduced-motion: reduce) {
      *, ::before, ::after {
        animation-duration: 0.01ms !important;
        animation-iteration-count: 1 !important;
        transition-duration: 0.01ms !important;
        scroll-behavior: auto !important;
      }
    }
    
    ${hasPoorConnection ? `
      *, ::before, ::after {
        animation-duration: 0.01ms !important;
        animation-iteration-count: 1 !important;
        transition-duration: 0.01ms !important;
        scroll-behavior: auto !important;
      }
      
      /* Further optimizations for slow connections */
      img[loading="lazy"] {
        opacity: 0;
        transition: opacity 0.5s;
      }
      img.loaded {
        opacity: 1;
      }
      
      /* Reduce quality of background images */
      .bg-image {
        image-rendering: optimizeSpeed;
      }
      
      /* Prevent parallax effects */
      .parallax {
        transform: none !important;
      }
    ` : ''}
  `;
  document.head.appendChild(style);
  
  // Preload the index.css file with low priority
  const preloadLink = document.createElement('link');
  preloadLink.rel = 'preload';
  preloadLink.as = 'style';
  preloadLink.href = '/src/index.css';
  preloadLink.setAttribute('importance', 'low');
  document.head.appendChild(preloadLink);
};

/**
 * Immediately start loading the CSS file with high priority
 */
const loadStylesheet = (href: string) => {
  const link = document.createElement('link');
  link.rel = 'stylesheet';
  link.href = href;
  document.head.appendChild(link);
};

/**
 * Main function to initialize the application
 */
const startApp = () => {
  // Time the startup
  const startTime = performance.now();
  
  // Inject critical CSS for better perceived performance
  injectCriticalStyles();
  
  // Add a hint to prefetch the index.css file
  const preloadLink = document.createElement('link');
  preloadLink.rel = 'preload';
  preloadLink.as = 'style';
  preloadLink.href = '/src/index.css';
  document.head.appendChild(preloadLink);
  
  // Function to render the React application
  const renderApp = () => {
    const rootElement = document.getElementById("root");
    if (!rootElement) return;
    
    try {
      // If the styles still haven't loaded after a threshold (500ms), start loading them
      // This is a fallback, as we don't want to show unstyled content for too long
      const elapsedTime = performance.now() - startTime;
      if (elapsedTime > 500 && !document.querySelector('link[rel="stylesheet"][href="/src/index.css"]')) {
        loadStylesheet('/src/index.css');
      }
      
      const root = createRoot(rootElement);
      
      // Use a micro-task to allow the browser to update the DOM before we start rendering
      queueMicrotask(() => {
        root.render(
          <Suspense fallback={<AppLoader />}>
            <App />
          </Suspense>
        );
        mark('app-render-complete');
        
        // Load the stylesheet shortly after the app renders if it hasn't been loaded already
        setTimeout(() => {
          if (!document.querySelector('link[rel="stylesheet"][href="/src/index.css"]')) {
            loadStylesheet('/src/index.css');
          }
        }, 50);
      });
    } catch (error) {
      console.error('Error rendering app:', error);
      
      // If there's an error, ensure the stylesheet is loaded
      loadStylesheet('/src/index.css');
    }
  };

  // Optimize the rendering sequence based on document readiness
  if (document.readyState === 'complete' || document.readyState === 'interactive') {
    // Document already loaded or interactive, render immediately
    // Use requestAnimationFrame to ensure we're not blocking the main thread
    requestAnimationFrame(() => {
      renderApp();
    });
  } else {
    // Wait for document to load, then render
    document.addEventListener('DOMContentLoaded', () => {
      // Use requestAnimationFrame to render after the browser has done layout and paint
      requestAnimationFrame(() => {
        renderApp();
      });
    });
  }
};

// Initialize the app
startApp();
